@@ -2,6 +2,8 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell #-}
@@ -93,14 +95,19 @@ module Text.Pretty.Simple
   , pStringOpt
   -- * 'OutputOptions'
   , OutputOptions(..)
+  , StringOutputStyle(..)
   , defaultOutputOptionsDarkBg
   , defaultOutputOptionsLightBg
   , defaultOutputOptionsNoColor
   , CheckColorTty(..)
   -- * 'ColorOptions'
-  -- $colorOptions
   , defaultColorOptionsDarkBg
   , defaultColorOptionsLightBg
+  , ColorOptions(..)
+  , Style(..)
+  , Color(..)
+  , Intensity(..)
+  , colorNull
   -- * Examples
   -- $examples
   ) where
@@ -112,16 +119,20 @@ import Control.Applicative
 #endif
 
 import Control.Monad.IO.Class (MonadIO, liftIO)
-import Data.Foldable (toList)
 import Data.Text.Lazy (Text)
-import Data.Text.Lazy.IO as LText
+import Prettyprinter (SimpleDocStream)
+import Prettyprinter.Render.Terminal
+      (Color (..), Intensity(Vivid,Dull), AnsiStyle,
+       renderLazy, renderIO)
 import System.IO (Handle, stdout)
 
 import Text.Pretty.Simple.Internal
-       (CheckColorTty(..), OutputOptions(..), defaultColorOptionsDarkBg,
-        defaultColorOptionsLightBg, defaultOutputOptionsDarkBg,
-        defaultOutputOptionsLightBg, defaultOutputOptionsNoColor,
-        hCheckTTY, expressionParse, expressionsToOutputs, render)
+       (ColorOptions(..), Style(..), CheckColorTty(..),
+        OutputOptions(..), StringOutputStyle(..),
+        convertStyle, colorNull,
+        defaultColorOptionsDarkBg, defaultColorOptionsLightBg,
+        defaultOutputOptionsDarkBg, defaultOutputOptionsLightBg,
+        defaultOutputOptionsNoColor, hCheckTTY, layoutString)
 
 -- $setup
 -- >>> import Data.Text.Lazy (unpack)
@@ -517,7 +528,9 @@ pHPrintStringOpt checkColorTty outputOptions handle str = do
     case checkColorTty of
       CheckColorTty -> hCheckTTY handle outputOptions
       NoCheckColorTty -> pure outputOptions
-  liftIO $ LText.hPutStrLn handle $ pStringOpt realOutputOpts str
+  liftIO $ do
+    renderIO handle $ layoutStringAnsi realOutputOpts str
+    putStrLn ""
 
 -- | Like 'pShow' but takes 'OutputOptions' to change how the
 -- pretty-printing is done.
@@ -527,13 +540,10 @@ pShowOpt outputOptions = pStringOpt outputOptions . show
 -- | Like 'pString' but takes 'OutputOptions' to change how the
 -- pretty-printing is done.
 pStringOpt :: OutputOptions -> String -> Text
-pStringOpt outputOptions =
-  render outputOptions . toList . expressionsToOutputs . expressionParse
+pStringOpt outputOptions = renderLazy . layoutStringAnsi outputOptions
 
--- $colorOptions
---
--- Additional settings for color options can be found in
--- "Text.Pretty.Simple.Internal.Color".
+layoutStringAnsi :: OutputOptions -> String -> SimpleDocStream AnsiStyle
+layoutStringAnsi opts = fmap convertStyle . layoutString opts
 
 -- $examples
 --
@@ -652,6 +662,36 @@ pStringOpt outputOptions =
 --         ]
 --     }
 --
+-- __Compactness options__
+--
+-- >>> pPrintStringOpt CheckColorTty defaultOutputOptionsDarkBg {outputOptionsCompact = True} "AST [] [Def ((3,1),(5,30)) (Id \"fact'\" \"fact'\") [] (Forall ((3,9),(3,26)) [((Id \"n\" \"n_0\"),KPromote (TyCon (Id \"Nat\" \"Nat\")))])]"
+-- AST []
+--     [ Def
+--         ( ( 3, 1 ), ( 5, 30 ) )
+--         ( Id "fact'" "fact'" ) []
+--         ( Forall
+--             ( ( 3, 9 ), ( 3, 26 ) )
+--             [ ( ( Id "n" "n_0" ), KPromote ( TyCon ( Id "Nat" "Nat" ) ) ) ]
+--         )
+--     ]
+--
+-- >>> pPrintOpt CheckColorTty defaultOutputOptionsDarkBg {outputOptionsCompactParens = True} $ B ( C [A, A] [B A, B (B (B A))] )
+-- B
+--     ( C
+--         [ A
+--         , A ]
+--         [ B A
+--         , B
+--             ( B ( B A ) ) ] )
+--
+-- __Initial indent__
+--
+-- >>> pPrintOpt CheckColorTty defaultOutputOptionsDarkBg {outputOptionsInitialIndent = 3} $ B ( B ( B ( B A ) ) )
+--    B
+--        ( B
+--            ( B ( B A ) )
+--        )
+--
 -- __Other__
 --
 -- Making sure the spacing after a string is correct.
@@ -665,3 +705,6 @@ pStringOpt outputOptions =
 --
 -- >>> pPrint "this string has non-printable characters: \x8 and \x9"
 -- "this string has non-printable characters: \x8 and \x9"
+--
+-- If you don't want non-printable characters to be escaped, take a look at
+-- 'outputOptionsStringStyle' and 'StringOutputStyle'.
